import 'package:source_gen/source_gen.dart';
import 'package:args/args.dart';

class ResourceConfig {
  static const List<String> IMAGE_EXTENSIONS = [
    ".png",
    ".jpg",
    ".jpeg",
    ".gif",
    ".webp",
    ".icon",
    ".bmp",
    ".wbmp",
    ".svg"
  ];

  final String resourcePath;
  final String imageClassName;
  final String fontClassName;
  final bool handleFontFile;
  final bool onlyAddFolder;
  final List<String> ignoreExtensions;
  final List<String> extraImageExtensions;
  final Map<String, String> extensionClassNameMapping;

  const ResourceConfig({
    this.resourcePath,
    this.imageClassName,
    this.fontClassName,
    this.handleFontFile = false,
    this.onlyAddFolder = true,
    this.ignoreExtensions,
    this.extraImageExtensions,
    this.extensionClassNameMapping,
  }) : assert(null != resourcePath, 'Resource path is null');

  static ResourceConfig fromAnnotation(ConstantReader annotation) {
    var resourcePath = annotation.peek('resourcePath')?.stringValue;
    if (resourcePath?.isNotEmpty ?? false) {
      var imageClassName =
          annotation.peek('imageClassName')?.stringValue ?? 'R_Image';
      var fontClassName =
          annotation.peek('fontClassName')?.stringValue ?? 'R_Font';
      var handleFontFile =
          annotation.peek('handleFontFile')?.boolValue ?? false;
      var onlyAddFolder = annotation.peek('onlyAddFolder')?.boolValue ?? true;
      var ignoreExtensions = annotation
          .peek('ignoreExtensions')
          ?.listValue
          ?.cast<String>()
          ?.checkDot();
      var extraImageExtensions = annotation
          .peek('extraImageExtensions')
          ?.listValue
          ?.cast<String>()
          ?.checkDot();
      var extensionClassNameMapping = annotation
          .peek('extensionClassNameMapping')
          ?.mapValue
          ?.cast<String, String>();
      extensionClassNameMapping?.keys?.toList()?.forEach((key) {
        var value = extensionClassNameMapping.remove(key);
        var finalKey =
            key.startsWith('.') ? key.toLowerCase() : '.$key'.toLowerCase();
        extensionClassNameMapping[finalKey] = value;
      });
      return ResourceConfig(
        resourcePath: resourcePath,
        imageClassName: imageClassName,
        fontClassName: fontClassName,
        handleFontFile: handleFontFile,
        onlyAddFolder: onlyAddFolder,
        ignoreExtensions: ignoreExtensions,
        extraImageExtensions: extraImageExtensions,
        extensionClassNameMapping: extensionClassNameMapping,
      );
    } else {
      throw Exception('Resource path is empty');
    }
  }

  static ResourceConfig fromArgs(ArgResults results) {
    if (results.wasParsed('resource-path')) {
      var resourcePath = results['resource-path'] as String;
      var imageClassName = results['image-class-name'] as String;
      var fontClassName = results['font-class-name'] as String;
      var handleFontFile = results['handle-font-file'] as bool;
      var onlyAddFolder = results['only-add-folder'] as bool;
      var ignoreExtensions =
          (results['ignore-extensions'] as String)?.split(',')?.checkDot();
      var extraImageExtensions =
          (results['extra-image-extensions'] as String)?.split(',')?.checkDot();
      var extensionClassNameMappingEntries =
          (results['extension-class-name-mapping'] as String)
              ?.split(',')
              ?.map((item) {
        var keyValue = item.trim().split(':');
        if (keyValue.length == 2) {
          var key = keyValue[0].trim();
          key = key.startsWith('.') ? key.toLowerCase() : '.$key'.toLowerCase();
          var value = keyValue[1].trim();
          return MapEntry(key, value);
        } else {
          throw Exception(
              'Invalid extension class name mapping value $keyValue');
        }
      });
      var extensionClassNameMapping = null != extensionClassNameMappingEntries
          ? Map.fromEntries(extensionClassNameMappingEntries)
          : null;
      return ResourceConfig(
        resourcePath: resourcePath,
        imageClassName: imageClassName,
        fontClassName: fontClassName,
        handleFontFile: handleFontFile,
        onlyAddFolder: onlyAddFolder,
        ignoreExtensions: ignoreExtensions,
        extraImageExtensions: extraImageExtensions,
        extensionClassNameMapping: extensionClassNameMapping,
      );
    } else {
      throw Exception('Resource path is empty');
    }
  }
}

extension _ExtensionDotChecker on List<String> {
  List<String> checkDot() {
    return map((extension) {
      extension = extension.trim();
      return extension.startsWith('.')
          ? extension.toLowerCase()
          : '.$extension'.toLowerCase();
    }).toList();
  }
}
